#!/usr/bin/env python3

import json
import os
import sys


import reader
import generator
import writer

CONTENT_DIR = '../content'
OUTPUT_DIR = '../output'
THEME_DIR = '../theme'


def main():
    print("Hello World")
    config = {}
    with open('../config.json') as f:
        config.update(json.loads(f.read()))
    with open('../lang.json') as f:
        config['lang'] = json.loads(f.read())
    with open(CONTENT_DIR + '/config.json') as f:
        config.update(json.loads(f.read()))
    print(config)

    mdreader = reader.MarkdownReader(config)
    pages = []
    directory = os.path.join(CONTENT_DIR, ".")
    parse_dir(directory, pages, mdreader)

    static_dirs = []
    for sdir in config['static_directories']:
        if "." in sdir:
            raise Exception("Illegal static directory name: ", sdir)
        static_dirs.append(CONTENT_DIR + "/"+ sdir)
    static_files = {}
    for sdir in static_dirs:
        read_static_dir(sdir, static_files)

    read_static_dir(THEME_DIR + '/static', static_files, [config['theme']['static_dir']])


    context = {}
    gen = generator.Generator(config, context)
    gen.generate_context(pages, static_files)

    wrt = writer.Writer(config, context, OUTPUT_DIR, THEME_DIR)

    gen.generate_output(wrt)



def read_static_dir(directory, static_files, subpath = []):
    print("static_dir: " + directory);
    for filename in os.listdir(directory):
        fp = os.path.join(directory, filename)
        if os.path.isfile(fp) and not filename.startswith("."):
            lpath = '/'.join(subpath + [filename])
            with open(fp, "rb") as f:
                static_files[lpath] = f.read()
            print("read: ", fp, " as: ", lpath)
        elif os.path.isdir(fp) and not filename.startswith("."):
            read_static_dir(fp, static_files, subpath + [filename])


def parse_dir(directory, pages, mdreader, subpath = []):
    print("parse_dir: " + directory);
    for filename in os.listdir(directory):
        f = os.path.join(directory, filename)
        if os.path.isfile(f) and filename.endswith(".md"):
            pages.append(mdreader.read_and_parse_file(f, subpath))
        elif os.path.isdir(f) and not filename.startswith("."):
            parse_dir(f, pages, mdreader, subpath + [filename])






    

if __name__ == '__main__':
    main()

