import datetime
from dateutil import parser as dtparser

class Page:

    #filename = None
    #subpath = None
    #raw = None
    #metadata = None
    #content = None
    #title = None
    #category = None
    #slug = None
    #lang = None
    #date_created = None
    #date_modified = None
    #status = None
    #authors = None
    #tags = None
    #template = None
    

    def __init__(self, filename, subpath, raw, metadata, content, title, category, slug, lang, date_created, date_modified, authors, last_modification_author, status, config):
        self.filename =  filename
        self.subpath =  subpath
        self.raw =  raw
        self.metadata =  metadata
        self.content =  content
        self.title =  title
        self.category =  category
        self.slug =  slug
        self.lang =  lang
        self.date_created =  Date(date_created, config)
        self.date_modified =  Date(date_modified, config)
        self.status =  status

        #self.config =  config

        # authors
        self.authors = []
        for local_part, domain, name in authors:
            self.authors.append(Author(local_part, domain, name, config))
        if last_modification_author:
            self.last_modification_author = Author(last_modification_author[0], last_modification_author[1], last_modification_author[2], config)
        else:
            self.last_modification_author = None
        #print(authors)
        #print(last_modification_author)


        # tags
        self.tags = []
        if 'tags' in metadata:
            tags = self.tags_str_to_list(metadata['tags'])
            for t in tags:
                if (t != category):
                    self.tags.append(t)
        self.tags.append(self.category) # the category is also a default tag

        # template
        if 'template' in metadata:
            self.template = metadata['template']
        else:
            self.template = config['theme']['default_template']

        # url
        self.url = self.lang + '/' + self.category + '/' + self.slug + ".html"

        # relevance
        self.relevance = {}
        self.relevance['is_relevant'] = False
        self.relevance['was_relevant'] = False
        self.relevance['will_be_relevant'] = False
        if 'relevant' in metadata:
            lrel = metadata['relevant']
            if not isinstance(lrel, dict):
                prio = lrel
                lrel = {}
                lrel['prio'] = prio

            if 'prio' in lrel:
                self.relevance['prio']= lrel['prio']
            else:
                self.relevance['prio']= 0

            start = Date("min", config)
            if 'start' in lrel:
                start = Date(lrel['start'], config)
            self.relevance['start'] = start

            end = Date("max", config)
            if 'end' in lrel:
                end = Date(lrel['end'], config)
            self.relevance['end'] = end

            build_date = Date("now", config)
            if (build_date > end):
                self.relevance['was_relevant'] = True
            elif (build_date < start):
                self.relevance['will_be_relevant'] = True
            else:
                self.relevance['is_relevant'] = True







    def tags_str_to_list(self, s):
        res = []
        l = s.split(',')
        for t in l:
            t = t.lower()
            t = t.lstrip()
            t = t.rstrip()
            if len(t) > 0:
                res.append(t)
        return res


    def __lt__(self, other):
        return self.date_modified < other.date_modified


class Date:
    def __init__(self, dt, config):
        if isinstance(dt, str):
            if dt == "min":
                dt = dtparser.parse("1970-01-01")
            elif dt == "max":
                dt = dtparser.parse("9999-01-01")
            elif dt == "now":
                dt = datetime.datetime.now()
            else:
                dt = dtparser.parse(dt)
        if isinstance(dt, datetime.date):
                dt = datetime.datetime(dt.year, dt.month, dt.day)
        self.dt = dt
        #self.config = config
        # TODO various formats
    def isoformat(self):
        return self.dt.isoformat()

    def __lt__(self, other):
        return self.dt.timestamp() < other.dt.timestamp()

class Author:
    def __init__(self, email_local_part, email_domain, name, config):
        self.email_local_part = email_local_part
        self.email_domain = email_domain
        self.email = email_local_part + '@' + email_domain
        self.name = name
        #self.config = config
        # TODO md5 hash etc.
