from jinja2 import Environment, FileSystemLoader

import os

class Writer:

    def __init__(self, config, context, output_dir, theme_dir):
        self.config = config
        self.context = context
        self.output_dir = output_dir
        self.theme_dir = theme_dir

        self.env = Environment(
            loader=FileSystemLoader(theme_dir + "/templates"),
            autoescape=False
        )

        print("templates: ", self.env.list_templates())

    def write_template(self, template, path, lang, extra_context):
        tmpl = self.env.get_template(template)

        pathsplit = path.split('/')
        #pathsplit.remove('.') # TODO remove all not just one
        #pathsplit.remove('..') # TODO remove all not just one

        siteurl = "."
        if self.config['relative_urls']:
            count = len(pathsplit) - 1
            for i in range(count):
                siteurl = siteurl + '/..'
        else:
            siteurl = config['siteurl']

        # render template
        context = {}
        context.update(self.context)
        context["config"] = self.config
        context["theme"] = self.config['theme']
        context["template"] = template
        context["t"] = self.config["lang"] # translate
        context["l"] = lang # current language
        context["path"] = path
        context["siteurl"] = siteurl
        context.update(extra_context)
        out = tmpl.render(context)

        # write file
        self.write_file(path, out)

    def write_file(self, path, out, mode="w"):
        # write to file
        fullpath = self.output_dir + '/' + path
        directory = os.path.dirname(fullpath)
        print("fullpath: ", fullpath)
        print("dir: ", directory)
        os.makedirs(directory, exist_ok=True)
        with open(fullpath, mode) as f:
            f.write(out)


